@@ -52,10 +52,11 @@ extern int set_res_value(const char *name, const char *value);
 extern void delete_res_value(const char *name);
 extern void save_resources(void);
 extern char *create_tmp_filename(void);
-extern int save_fasta_file(const char *fname, char **seq, char **comments,
-	char **seqname, int totseqs, int *eachlength, region *region_used, int *sel_seqs, int tot_sel_seqs);
+extern int save_fasta_file(const char *fname, char **seq, char **comments, char **seqname, 
+	int totseqs, int *eachlength, region *region_used, int *sel_seqs, int tot_sel_seqs, int spaces_in_names);
 extern void direct_help_callback(Fl_Widget *wgt, void *data);
 extern int compute_size_params(SEA_VIEW *view, int force_recompute);
+extern char **prepcolranks_by_codon(char **seq, int totseqs, int maxlen, int *eachlength, char **comments);
 #ifdef WIN32
 extern "C" { int mysystem(const char *command); }
 #endif
@@ -289,7 +290,7 @@ for(num = 0; num < view->tot_seqs; num++) if(tmpselseqs[num]) total++;
 view->cursor_in_comment = FALSE;
 view->cursor_seq = view->first_seq;
 /* lecture de l'alignement multiple produit par clustalw */
-num = read_fasta_align(fname, &seq, &seqname, &comments, &pheader, &err_message);
+num = read_fasta_align(fname, &seq, &seqname, &comments, &pheader, &err_message, FALSE);
 for(i = 0; i < num; i++) free(seqname[i]);
 if(num > 0) { free(seqname); free(comments); }
 if( num != total) {
@@ -1087,7 +1088,7 @@ void profile_align(SEA_VIEW *view)
 	sprintf(fname1, "%s.aligned", base_fname);
 	/* lecture de l'alignement multiple */
 	if( status == 0 ) {
-		num = read_fasta_align(fname1, &seq, &seqname, &comments, &pheader, &err_message);
+		num = read_fasta_align(fname1, &seq, &seqname, &comments, &pheader, &err_message, FALSE);
 		if(num > 0) { 	
 			for(i = 0; i < num; i++) free(seqname[i]);
 			free(seqname); free(comments); 
@@ -1214,7 +1215,7 @@ int save_fasta_file_no_gaponly(char *fname, char **sequence, int tot_seqs, int *
 		}
 	if(insegment) segment->fin = l;
 	segment->next = NULL;
-	status = save_fasta_file(fname, sequence, NULL, seqname, tot_seqs, each_length, &maregion, sel_seqs, 1);
+	status = save_fasta_file(fname, sequence, NULL, seqname, tot_seqs, each_length, &maregion, sel_seqs, 1,0);
 	segment = maregion.list;
 	do	{
 		next = segment->next;
@@ -1252,12 +1253,21 @@ void unalign(SEA_VIEW *view)
 		view->sequence[num] = tmpseq;
 		view->each_length[num] = strlen(tmpseq);
 		if(view->numb_gc > 1) {
-			for(i = 0; i < view->each_length[num]; i++) {
-				res = view->sequence[num][i];
-				view->col_rank[num][i] = (char)calc_color_function( res );
+			if( view->curr_colors == view->codoncolors) {
+				char **tmp = prepcolranks_by_codon(&view->sequence[num], 1, view->max_seq_length, 
+												   &view->each_length[num], &view->comments[num]);
+				free(view->col_rank[num]);
+				view->col_rank[num] = tmp[0];
+				free(tmp);
+				}
+			else {
+				for(i = 0; i < view->each_length[num]; i++) {
+					res = view->sequence[num][i];
+					view->col_rank[num][i] = (char)calc_color_function( res );
+					}
+				}
 			}
 		}
-	}
 	newlength = 0;
 	for(num = 0; num < view->tot_seqs; num++) newlength = FL_max(newlength, view->each_length[num]);
 	view->seq_length = newlength;
