@@ -1,4 +1,5 @@
 #include "seaview.h"
+#include <ctype.h>
 #include <FL/Fl_Color_Chooser.H>
 #include <FL/Fl_Slider.H>
 #include <FL/Fl_Check_Button.H>
@@ -15,6 +16,8 @@ void permanent_cb(Fl_Widget *obj, void *data);
 void reset_color_callback(Fl_Widget *obj, void *data);
 void accept_color_callback(Fl_Widget *obj, void *data);
 void reset_callback(Fl_Widget *obj, void *data);
+void codon_color_chooser_cb(Fl_Widget *obj, void *data);
+int is_view_valid(SEA_VIEW *view);
 
 
 /* private global variables */
@@ -22,7 +25,7 @@ static Fl_Button *A, *C, *G, *T;
 static Fl_Slider *bgrnd, *s_bgrnd;
 static Fl_Input *protgroups, *s_protgroups, *movekeys;
 static Fl_Button **protcol;
-static Fl_Check_Button *lowercase, *inverted;
+static Fl_Check_Button *lowercase, *inverted, *fastaspaces;
 static Fl_Choice *pdfformat, *saveformat, *alignment_algo;
 static Fl_Value_Input *pdffontsize;
 static Fl_Value_Input *phylipwidnames;
@@ -43,19 +46,32 @@ extern color_choice prep_aa_color_code(char *list_std, char *list_alt,
 extern int set_res_value(const char *name, const char *value);
 extern int save_resources(void);
 extern void minuscules(char *);
-extern char *get_res_value(char *name, char *def_value);
+extern char *get_res_value(const char *name, const char *def_value);
+extern char aminoacids[]; // from misc_acnuc.c
+
+
+class coloraas : public Fl_Widget {
+	void draw(void);
+public:
+	int codoncolors[22];
+	coloraas(int x,int y,int w,int h, int *in_codoncolors) : Fl_Widget(x,y,w,h) 
+	{
+		memcpy(codoncolors, in_codoncolors, sizeof(codoncolors));
+	}
+};
 
 
 void custom_callback(Fl_Widget *obj, void *data)
 {
 static Fl_Window *win = NULL;
 static Fl_Button *permanent, *accept;
+static coloraas *mycoloraas;
 int x, y, w, h, c;
 SEA_VIEW *view = (SEA_VIEW *)data;
 Fl_Button *reset;
 
 if(win == NULL) {
-	win = new Fl_Window(310, 530, "Customization");
+	win = new Fl_Window(310, 600, "Customization");
 	
 	x = 9; y = 20; w = 160; h = 20;
 	bgrnd = new Fl_Slider(x,y,w,h, "background grey");
@@ -131,10 +147,22 @@ if(win == NULL) {
 	reset->callback(reset_callback, protgroups);
 	new Fl_Box(FL_DOWN_FRAME, 5, topbox,
 		win->w() - 8, y + reset->h() + 5 - topbox, "");
-
+	
+	topbox = reset->y() + reset->h() + 10;
+	Fl_Box *frame = new Fl_Box(FL_DOWN_FRAME, 5, topbox,
+			   win->w() - 8, 50, "");
+	Fl_Box *box = new Fl_Box(FL_NO_BOX, 0, topbox, win->w(), 20, "Synonymous codon colors");
+	box->align(FL_ALIGN_CENTER);
+	mycoloraas = new coloraas(10, topbox + 20, win->w() - 10, 20, view->codoncolors);
+	mycoloraas->labelfont(FL_COURIER);
+	mycoloraas->labelsize(18);
+	Fl_Button *codons = new Fl_Button(win->w() - 65, topbox + 20, 60, 20, "Change");
+	codons->callback(codon_color_chooser_cb, mycoloraas);
+	codons->clear_visible_focus();
+	
 	x = A->x();
-	y += protcol[1]->h() + 30; 
-	movekeys = new Fl_Input(x,y, 50, 20, "right-left movement keys");
+	y = frame->y() + frame->h() + 25;
+	movekeys = new Fl_Input(x,y, 50, 20, "right-left move keys");
 	movekeys->align(FL_ALIGN_TOP | FL_ALIGN_LEFT);
 	reset = new Fl_Button(x + movekeys->w() + 5, y, 50, 20, "reset");
 	reset->color(FL_LIGHT2);
@@ -153,6 +181,9 @@ if(win == NULL) {
 
 	y += lowercase->h(); 
 	inverted = new Fl_Check_Button(x,y,140,20, "inverted");
+  
+	y += inverted->h();
+	fastaspaces = new Fl_Check_Button(x,y,170,20, "spaces in FASTA names");
 	
 	x = win->w() - w - 5;
 	y = movekeys->y() + movekeys->h() + 20;
@@ -167,15 +198,17 @@ if(win == NULL) {
 	pdffontsize = new Fl_Value_Input(x+w/2,y,w/2,h, PDF_OR_PS" fontsize");
 	pdffontsize->align(FL_ALIGN_TOP_RIGHT);
 	pdffontsize->labelsize(10);
-
-	y = pdfformat->y() + pdfformat->h() + 20;
-	phylipwidnames = new Fl_Value_Input(x,y,w,h, "phylip names width");
-	phylipwidnames->align(FL_ALIGN_TOP);
 	
 	x = A->x();
-	y = inverted->y() + inverted->h() + 20; 
+	y = fastaspaces->y() + fastaspaces->h() + 20; 
 	alignment_algo = new Fl_Choice(x,y,w,h, "alignment algorithm");
 	alignment_algo->align(FL_ALIGN_TOP);
+  
+	y = alignment_algo->y();
+	x = pdfformat->x();
+	phylipwidnames = new Fl_Value_Input(x,y,w,h, "phylip names width");
+	phylipwidnames->align(FL_ALIGN_TOP);
+
 	x = A->x();
 	y += alignment_algo->h() + 10;
 	h = 40;
@@ -197,8 +230,9 @@ if(win == NULL) {
 	win->resizable(NULL);
 	}
 
-accept->callback(accept_cb, view);
-permanent->callback(permanent_cb, view);
+mycoloraas->user_data(view);
+accept->callback(accept_cb, mycoloraas);
+permanent->callback(permanent_cb, mycoloraas);
 bgrnd->value( view->DNA_obj->color() );
 bgrnd->color( view->DNA_obj->color() );
 s_bgrnd->value( view->DNA_obj->color2() );
@@ -240,6 +274,7 @@ for(c=1; c < max_protcolors; c++) {
 movekeys->value( view->movekeys );
 lowercase->value(view->allow_lower);
 inverted->value(view->inverted_colors);
+fastaspaces->value(view->spaces_in_fasta_names);
 pdfformat->value(printout_pageformat);
 pdffontsize->value(printout_fontsize);
 phylipwidnames->value(view->phylipwidnames);
@@ -269,13 +304,17 @@ obj->window()->hide();
 
 void accept_cb(Fl_Widget *obj, void *data)
 {
-SEA_VIEW *view = (SEA_VIEW *)data;
+coloraas *mycoloraas = (coloraas *)data;
+SEA_VIEW *view = (SEA_VIEW *)mycoloraas->user_data();
+if(!is_view_valid(view)) return;
 Fl_Menu_Button *menu_props = (Fl_Menu_Button *)view->bouton_props;
 props_menu_parts *props_parts = (props_menu_parts *)menu_props->user_data();
 char *p;
 Fl_Menu_Item *item;
 
 my_watch_cursor(view->dnawin);
+memcpy(view->codoncolors, mycoloraas->codoncolors, sizeof(mycoloraas->codoncolors));
+	
 view->DNA_obj->color((int)(bgrnd->value() + 0.5));
 view->DNA_obj->color2((int)(s_bgrnd->value() + 0.5));
 view->DNA_obj->parent()->color( view->DNA_obj->color() );
@@ -329,6 +368,8 @@ menu_props->value(props_parts->inverted);
 item->do_callback(menu_props);
 if(view->inverted_colors) item->set();
 else item->clear();
+  
+view->spaces_in_fasta_names = fastaspaces->value();
 
 printout_pageformat = (paperformat)pdfformat->value();
 printout_fontsize = (int)pdffontsize->value();
@@ -353,9 +394,10 @@ fl_reset_cursor(view->dnawin);
 
 void permanent_cb(Fl_Widget *obj, void *data)
 {
-char txt[150];
-int v;
+char txt[150], *p;
+int v, i;
 uchar r, g, b;
+coloraas *mycoloraas = (coloraas *)data;
 
 accept_cb(obj, data);
 
@@ -390,6 +432,7 @@ set_res_value("protcolors", txt);
 set_res_value("movekeys", movekeys->value() );
 set_res_value("lowercase", lowercase->value() ? "true" : "false");
 set_res_value("inverted", inverted->value() ? "true" : "false");
+set_res_value("spacesinfastanames", fastaspaces->value() ? "true" : "false");
 set_res_value("printoutpageformat", printout_pageformat == A4 ? "A4" : "LETTER" );
 sprintf(txt, "%d", (int)pdffontsize->value() );
 set_res_value("printoutfontsize",  txt);
@@ -404,6 +447,14 @@ set_res_value("save", txt );
 v = alignment_algo->value();
 sprintf(txt, "%d", v );
 set_res_value("alignment", txt);
+	
+p = txt;
+for(i = 0; i < 21; i++) {
+	sprintf(p, "%c:%d,", aminoacids[i], mycoloraas->codoncolors[i+1]);
+	p += strlen(p);
+	}
+*(p-1) = 0;
+set_res_value("codoncolors", txt);
 
 save_resources();
 obj->window()->hide();
@@ -517,3 +568,287 @@ void slider_cb(Fl_Widget *obj, void *data)
 Fl_Valuator *v = (Fl_Valuator *)obj;
 v->color( (int)(v->value() + 0.5) );
 }
+
+
+class colorcell : public Fl_Widget {
+    FL_EXPORT void draw(void);
+    FL_EXPORT int handle(int);
+public:
+    FL_EXPORT colorcell(int x,int y,int w,int h) :
+	Fl_Widget(x,y,w,h,NULL) {
+		this->copy_label(" ");
+	}
+};
+
+
+class codon_color_chooser : public Fl_Window
+	{
+	public:
+		Fl_Group *g;
+		colorcell *selected;
+		coloraas *mycoloraas;
+		codon_color_chooser(coloraas *mycoloraas);
+	private:
+		static void cancel_cb(Fl_Widget *ob, void *data);
+		static void new_colors_cb(Fl_Widget *w, void *data);
+		static void reset_cb(Fl_Widget *w, void *data);
+		static void defaults_cb(Fl_Widget *w, void *data);
+	};
+
+
+int colorcell::handle(int event) 
+{
+	char *aa, *p;
+	int key, before, i;
+	Fl_Group *g;
+	colorcell *cell;
+	codon_color_chooser *mychooser = (codon_color_chooser *)this->window();
+
+    switch (event)
+    {
+		case FL_RELEASE:
+			if(mychooser->selected) {
+				mychooser->selected->redraw();
+				}
+			mychooser->selected = this;
+			this->redraw();
+			this->take_focus();
+			break;
+		case FL_KEYBOARD:
+			if( Fl::event_state(myFL_CTRL) ) return 0; // can be shortcut of other widgets
+			if(Fl::event_length() == 0) break;
+			key = Fl::event_text()[0];
+			key = toupper(key);
+			if((aa = strchr(aminoacids, key)) == NULL) break;
+			if(*aa == 'X') break;
+			p = (char *)this->label();
+			before = *p;
+			*p = *aa;
+			this->redraw();
+			g = parent();
+			for(i=0; i < 256; i++) {
+				cell = (colorcell *)g->child(i);
+				p = (char *)cell->label();
+				if(*p == key && cell != this) {
+					*p = before;
+					cell->redraw();
+					break;
+					}
+				}
+			break;
+		case FL_FOCUS:
+			break;
+		default:
+			return 0;
+	}
+	return 1;
+}
+
+
+void colorcell::draw(void) 
+{
+	codon_color_chooser *mychooser = (codon_color_chooser *)this->window();
+	fl_color(color());
+	fl_rectf(x(), y(), w(), h());
+	fl_color(FL_BLACK);
+	if(*label() != ' ') {
+		fl_font(labelfont(), labelsize());
+		fl_draw(label(), 1, x() + w()/2 - (int)fl_width(label())/2, y() + h()/2 + fl_size()/3 );
+		}
+	if(mychooser->selected == this) {
+		fl_line_style(0, 3);
+		fl_rect(x() + 2, y() + 2, w()-4, h()-4);
+		fl_line_style(0, 0);
+		}
+}
+
+
+void decode_codon_colors(int *colors)
+// puts desired colorindex in colors[1..21] in the order of aminoacids table
+{
+	const char *q;
+	char *p, *codestring;
+	int i, val, retval = 0;
+	
+	codestring = get_res_value("codoncolors", NULL);
+	if(codestring != NULL) {
+//codestring contains R:10,W:155,*:5,....
+		p = codestring;
+		for(i = 1; i <= 21; i++) {
+			q = strchr(aminoacids, *p);
+			if(q == NULL || *q == 'X') {
+				p = strchr(p, ',');
+				if(p == NULL) break;
+				p++;
+				continue;
+			}
+			p += 2;
+			sscanf(p, "%d", &val);
+			colors[q - aminoacids + 1] = val;
+			retval++;
+			p = strchr(p, ',');
+			if(p == NULL) break;
+			p++;
+			}
+		}
+	if(retval != 21) {//absence or incomplete codoncolors preference
+		for(i = 1; i <= 21; i++) colors[i] = i + 72;
+		return;
+	}
+}
+
+
+void codon_color_chooser::defaults_cb(Fl_Widget *w, void *data)
+{
+	int aa, colrank, i;
+	char *p;
+	codon_color_chooser *mychooser = (codon_color_chooser *)w->window();
+	colorcell *cell;
+	for(i=0; i < 256; i++) {
+		cell = (colorcell *)mychooser->g->child(i);
+		p = (char *)cell->label();
+		*p = ' ';
+		}
+	for(aa=1; aa <= 21; aa++) {
+		colrank = aa + 72;
+		cell = (colorcell *)mychooser->g->child(colrank);
+		p = (char *)cell->label();
+		*p = aminoacids[aa-1];
+	}
+	mychooser->selected = NULL;
+	mychooser->g->redraw();
+}
+
+
+void codon_color_chooser::cancel_cb(Fl_Widget *ob, void *data)
+{
+	codon_color_chooser *mychooser = (codon_color_chooser *)(ob->window() ? ob->window() : ob);
+	mychooser->hide();
+	Fl::delete_widget(mychooser);
+}
+
+
+void codon_color_chooser::new_colors_cb(Fl_Widget *w, void *data)
+{
+	codon_color_chooser *mychooser = (codon_color_chooser *)w->window();
+	char *aa;
+	int rank, i;
+	colorcell *cell;
+	
+	for(i=0; i < 256; i++) {
+		cell = (colorcell *)mychooser->g->child(i);
+		aa = (char *)cell->label();
+		if(*aa == ' ') continue;
+		rank = strchr(aminoacids, *aa) - aminoacids;
+		mychooser->mycoloraas->codoncolors[rank+1] = i;
+		}
+	mychooser->mycoloraas->redraw();
+	mychooser->do_callback();
+}
+
+
+void codon_color_chooser::reset_cb(Fl_Widget *w, void *data)
+{
+	codon_color_chooser *mychooser = (codon_color_chooser *)w->window();
+	char *p;
+	int aa, colrank, colors[22];
+	colorcell *cell;
+	
+	for(colrank=0; colrank < 256; colrank++) {
+		cell = (colorcell *)mychooser->g->child(colrank);
+		p = (char *)cell->label();
+		*p = ' ';
+	}
+	decode_codon_colors(colors);
+	//we have now a complete list of colors
+	for(aa = 1; aa <= 21; aa++) {
+		cell = (colorcell *)mychooser->g->child(colors[aa]);
+		p = (char *)cell->label();
+		*p = aminoacids[aa-1];
+		}
+	mychooser->selected = NULL;
+	mychooser->g->redraw();
+}
+
+
+void codon_color_chooser_cb(Fl_Widget *obj, void *data)
+{
+	codon_color_chooser *mychooser = new codon_color_chooser( (coloraas *)data );
+	mychooser->show();
+}
+
+
+codon_color_chooser::codon_color_chooser(coloraas *in_coloraas) : Fl_Window(330, 414)
+{
+	int i, j;
+	colorcell *box;
+	
+	mycoloraas = in_coloraas;
+	label("Synonymous codon color chooser");
+	set_modal();
+	Fl_Box *b = new Fl_Box(FL_NO_BOX, 0, 0, this->w(), 60,
+			"Click on color cell, and type desired\none-letter amino acid (or * for stop codons).");
+	b->align(FL_ALIGN_CENTER);
+	g = new Fl_Group(5, 65, 320, 320);
+	for(i=0; i < 16; i++) {
+		for(j=0; j < 16; j++) {
+			box = new colorcell( 5 + j*20, 65 + i * 20, 20, 20);
+			box->box(FL_FLAT_BOX);
+			box->color(16*i + j);
+			box->align(FL_ALIGN_CENTER);
+			box->labelfont(FL_COURIER_BOLD);
+			box->labelsize(14);
+			}
+		}
+	g->end();
+//fill cells with adequate aas
+	for(i = 1; i <= 21; i++) {
+		colorcell *cell = (colorcell *)g->child(mycoloraas->codoncolors[i]);
+		char *p = (char *)cell->label();
+		*p = aminoacids[i-1];
+	}
+	
+	Fl_Button *cancel = new Fl_Button(5, 390, 60, 20, "Cancel");
+	cancel->callback(cancel_cb, NULL);
+	cancel->tooltip("don't change anything");
+	cancel->shortcut(myFL_CTRL | 'w');
+	Fl_Button *reset = new Fl_Button(100, 390, 60, 20, "Reset");
+	reset->callback(reset_cb, NULL);
+	reset->tooltip("back to previous color set");
+	Fl_Button *defaults = new Fl_Button(170, 390, 70, 20, "Defaults");
+	defaults->callback(defaults_cb, NULL);
+	defaults->tooltip("use seaview's default color set");
+	Fl_Button *ok = new Fl_Button(275, 390, 50, 20, "OK");
+	ok->callback(new_colors_cb, NULL);
+	ok->tooltip("accept new color set");
+	end();
+	callback(cancel_cb, NULL);
+	selected = NULL;
+}
+
+
+void coloraas::draw(void)
+{
+	int i, wl;
+	fl_font(this->labelfont(), this->labelsize());
+	wl = (int)(fl_width("A") + 0.5);
+	for(i=0; i < 21; i++) {
+		fl_color(this->codoncolors[i + 1]);
+		fl_rectf(x() + i * wl, y(), wl, h());
+		}
+	fl_color(FL_BLACK);
+	fl_draw(aminoacids, 21, x() , y() + h()/2 + fl_size()/3 );
+}
+
+
+int is_view_valid(SEA_VIEW *view)
+{
+	Fl_Window *w = Fl::first_window();
+	while(w != NULL) {
+		const char *wclass = w->xclass();
+		if(wclass != NULL && strcmp(wclass, SEAVIEW_WINDOW) == 0 && w->user_data() == view) return TRUE;
+		w = Fl::next_window(w);
+		}
+	return FALSE;
+}
+
