@@ -174,11 +174,11 @@ void load_seq_dialog(SEA_VIEW *view)
 static Fl_Window *load_form;
 static int first = TRUE;
 static Fl_Input *seq_input, *seq_name_input;
+static view_name_struct view_name;
 
 if(first) {
 	Fl_Widget *obj;
 	int fin, width;
-	static view_name_struct view_name;
 	first = FALSE;
 	load_form = new Fl_Window(490,530);
 	load_form->box(FL_FLAT_BOX);
@@ -262,15 +262,15 @@ if(first) {
 	
 	load_form->resizable(seq_input);
 	
-	view_name.view = view;
 	view_name.seq_name_field = seq_name_input;
 	view_name.seq_field = seq_input;
 	load_form->end();
-    load_form->position((Fl::w() - load_form->w())/2, 
+        load_form->position((Fl::w() - load_form->w())/2, 
 		(Fl::h() - load_form->h())/2);
 	}
 seq_name_input->value("");
 seq_input->value("");
+view_name.view = view;
 #if defined(__APPLE__)
 	seq_input->value("\n\n\n\n\n\nType sequence or paste it with cmd-V\n\n\n");
 	seq_input->position(seq_input->size(), 0);
@@ -739,11 +739,63 @@ return FALSE;
 }
 
 
+char calc_site_consensus(int *freqs, int dernier, int threshold, char *residues, char unknown, int total, 
+						 int protein, int allowgaps, int iupac)
+{
+	char retval;
+	int maxi = 0, num, kind, last;
+	maxi = freqs[1];
+	kind = 1;
+	if(protein) last = dernier;
+	else last = 5;
+	for(num = 2; num < last; num++) {
+		if(freqs[num] > maxi) {
+			maxi = freqs[num];
+			kind = num;
+			}
+		}
+	if(allowgaps && !protein) {
+		num = dernier - 1;//check if '-' has the highest freq
+		if(freqs[num] > maxi) {
+			maxi = freqs[num];
+			kind = num;
+		}
+	}
+	if(!protein) total -= freqs[16];//DNA: don't count uninformative N's for frequency calculation
+	float minval = total * (threshold / 100.);
+	if(iupac && !protein ) { // for DNA/RNA with IUPAC codes
+		if( maxi >= minval ) retval = residues[kind - 1];
+		else if(freqs[1] + freqs[2] + freqs[6] >= minval) retval = 'M';//AC
+		else if(freqs[1] + freqs[3] + freqs[7] >= minval) retval = 'R';//AG
+		else if(freqs[1] + freqs[4] + freqs[5] + freqs[8] >= minval) retval = 'W';//AT(U)
+		else if(freqs[2] + freqs[3] + freqs[9] >= minval) retval = 'S';//CG
+		else if(freqs[2] + freqs[4] + freqs[5] + freqs[10] >= minval) retval = 'Y';//CT(U)
+		else if(freqs[3] + freqs[4] + freqs[5] + freqs[11] >= minval) retval = 'K';//GT(U)
+		else if(freqs[1] + freqs[2] + freqs[3] + freqs[6] + 
+				freqs[9] + freqs[7] + freqs[12] >= minval) retval = 'V';//ACG
+		else if(freqs[1] + freqs[2] + freqs[4] + freqs[5] + 
+				freqs[6] + freqs[8] + freqs[10] + freqs[13] >= minval) retval = 'H';//ACT(U)
+		else if(freqs[1] + freqs[3] + freqs[4] + freqs[5] + 
+				freqs[7] + freqs[8] + freqs[11] + freqs[14] >= minval) retval = 'D';//AGT(U)
+		else if(freqs[2] + freqs[3] + freqs[4] + freqs[5] + 
+				freqs[9] + freqs[10] + freqs[11] + freqs[15] >= minval) retval = 'B';//CGT(U)
+		else retval = 'N';
+		}
+	else {
+		if( maxi >= minval )
+			retval = residues[kind - 1];
+		else
+			retval = unknown;
+		}
+	return retval;
+}
+	
+
 char *cre_consensus(SEA_VIEW *view, char *newname)
 {
 char *newseq, *p, *residues, unknown;
-int pos, num, total, kind, dernier, maxi, vu;
-static char dna_residues[]="ACGTU-";
+int pos, num, total, kind, dernier, vu;
+static char dna_residues[]="ACGTUMRWSYKVHDBN-";
 static const char prot_residues[] = "EDQNHRKILMVAPSGTFYWC-";
 static int freqs[30];
 
@@ -776,21 +828,11 @@ for (pos = 0; pos < view->seq_length; pos++) {
 	if(total == 0)
 		newseq[pos] = '-';
 	else	{
-		maxi = 0;
-		for(num = 0; num < dernier; num++) {
-			if(freqs[num] > maxi) {
-				maxi = freqs[num]; kind = num;
-				}
-			}
-		if(kind == 0)
-			newseq[pos] = unknown;
-		else if( maxi >= total * (view->consensus_threshold / 100.) )
-			newseq[pos] = residues[kind - 1];
-		else
-			newseq[pos] = unknown;
+		newseq[pos] = calc_site_consensus(freqs, dernier, view->consensus_threshold, residues, unknown, 
+										  total, view->protein, view->consensus_allowgaps, view->consensus_allowiupac);
 		}
 	newseq[pos + 1] = 0;
 	}
-strcpy(newname, "Consensus");
+sprintf(newname, "Consensus%d", view->consensus_threshold);
 return newseq;
 }
