@@ -12,7 +12,7 @@ extern int printout_block, printout_fontsize, printout_black, printout_vary;
 extern paperformat printout_pageformat;
 extern char *prepare_ps_or_pdf_font(int font_num);
 extern paperformat printout_pageformat;
-extern int pdf_or_ps_plot, page_count, ps_width; 
+extern int pdf_or_ps_plot, ps_width; 
 extern double physx, physy, maxx, nexty;
 extern char *end_br_length;
 extern void mem_plot(FD_nj_plot *fd_nj_plot, struct noeud *pere, struct noeud *centre, double currx,
@@ -425,7 +425,7 @@ void pdf_tree_plot(FD_nj_plot *fd_nj_plot)
 		
 		/* recouvre = paper_h / 30; */
 		recouvre = 0;
-		sheets_h = ((int)paper_h - recouvre) * page_count + recouvre;
+		sheets_h = ((int)paper_h - recouvre) * fd_nj_plot->page_count + recouvre;
 		offset = (int)paper_h - recouvre;
 		physx = ps_width;
 		physy = sheets_h;
@@ -441,20 +441,21 @@ void pdf_tree_plot(FD_nj_plot *fd_nj_plot)
 			mem_plot(fd_nj_plot, NULL, fd_nj_plot->racine, currx, &curry);
 		else
 			mem_plot(fd_nj_plot, fd_nj_plot->subtree_ascend, fd_nj_plot->subtree_center, currx, &curry);
-		for(page = 0; page < page_count ; page++) {
+		for(page = 0; page < fd_nj_plot->page_count ; page++) {
 			PDF_begin_page_ext(pdf, 
 							   printout_pageformat == A4 ? a4_width : letter_width,
 							   printout_pageformat == A4 ? a4_height : letter_height,
 							   "");
 			PDF_setfont(pdf, t_font, 12);
-			sprintf(bigline, "%s  %s  Page %d of %d", extract_filename(fd_nj_plot->tree_name), ctime(&heure), page+1, page_count);
+			sprintf(bigline, "%s  %s  Page %d of %d", extract_filename(fd_nj_plot->tree_name), ctime(&heure), page+1, fd_nj_plot->page_count);
 			PDF_show_xy(pdf, bigline, 40, paper_h+65);
 			
+			PDF_setlinecap(pdf, 1);
 			PDF_setlinewidth(pdf, 1);
 			PDF_moveto(pdf, 40, 40); PDF_lineto(pdf, ps_width + 60, 40); PDF_lineto(pdf, ps_width + 60, paper_h+60);
 			PDF_lineto(pdf, 40, paper_h+60); 
 			PDF_closepath(pdf); PDF_clip(pdf);
-			PDF_translate(pdf, 0, - (page_count - page - 1) * offset);
+			PDF_translate(pdf, 0, - (fd_nj_plot->page_count - page - 1) * offset);
 			PDF_setfont(pdf, pdf_font, pdf_font_size);
 			PDF_translate(pdf, 50, 50);
 			PDF_setcolor(pdf, "stroke", "gray", 0.7, 0,0,0);
