@@ -260,7 +260,7 @@ int run_external_prog_in_pseudoterm(char *cmd, const char *dialogfname, const ch
 	HANDLE ChildInput, ChildStdoutRd, ChildStdoutWr, stdouthandle = NULL;
 	SECURITY_ATTRIBUTES saAttr; 
 	PROCESS_INFORMATION pi; 
-	STARTUPINFO si;
+	STARTUPINFOW si;
 	char *p;
 	
 	if(dialogfname != NULL) {
@@ -290,14 +290,19 @@ int run_external_prog_in_pseudoterm(char *cmd, const char *dialogfname, const ch
 	WriteFile(ChildStdoutWr, "\n ", 1, &written, NULL);
 	SetHandleInformation(ChildStdoutRd, HANDLE_FLAG_INHERIT, 0);
 	ZeroMemory( &pi, sizeof(PROCESS_INFORMATION) );
-	ZeroMemory( &si, sizeof(STARTUPINFO) );
-	si.cb = sizeof(STARTUPINFO);
+	ZeroMemory( &si, sizeof(STARTUPINFOW) );
+	si.cb = sizeof(STARTUPINFOW);
 	si.hStdError = ChildStdoutWr;
 	si.hStdOutput = stdouthandle ? stdouthandle : ChildStdoutWr;
 	if(dialogfname != NULL) si.hStdInput = ChildInput;
 	else si.hStdInput = GetStdHandle(STD_INPUT_HANDLE);
 	si.dwFlags |= STARTF_USESTDHANDLES;
-	int retval = CreateProcess(NULL, cmd, 0,0,TRUE,CREATE_NO_WINDOW,0,0, &si, &pi);
+//conversion of utf-8 cmd into a WCHAR string
+	int wlen = MultiByteToWideChar(CP_UTF8, 0, cmd, -1, NULL, 0);
+	WCHAR *wcmd = new WCHAR[wlen];
+	MultiByteToWideChar(CP_UTF8, 0, cmd, -1, wcmd, wlen);
+	int retval = CreateProcessW(NULL, wcmd, 0,0,TRUE,CREATE_NO_WINDOW,0,0, &si, &pi);
+	delete wcmd;
 	CloseHandle(ChildStdoutWr);
 	if(stdouthandle != NULL) CloseHandle(stdouthandle);
 	if(dialogfname != NULL) CloseHandle(ChildInput);
