@@ -405,10 +405,12 @@ raa_db_access *open_acnuc_socket(Fl_Window *w)
 	char *server_ip;
 	int port, err;
 	char *url = get_res_value("racnuc", def_acnuc_server);
-	raa_decode_address(url, &server_ip, &port, NULL);
-	my_watch_cursor(w);
-	err = raa_open_socket(server_ip, port , (char *)"seaview", &raa); 
-	fl_reset_cursor(w);
+	err = raa_decode_address(url, &server_ip, &port, NULL);
+	if(!err) {
+		my_watch_cursor(w);
+		err = raa_open_socket(server_ip, port , (char *)"seaview", &raa); 
+		fl_reset_cursor(w);
+		}
 	if(err) fl_alert("Can't connect to remote acnuc server:\n%s", url);
 	return raa;
 }
