@@ -1,6 +1,11 @@
 #include "seaview.h"
+#include <FL/Fl_Round_Button.H>
 
 
+enum position_mask {custom_mask = -1, position_1, position_2, position_3, position_12};
+const char *position_mask_names[4] = {"First codon pos.", "Second codon pos.",
+"Third codon pos.", "1st+2nd codon pos."};
+
 /* prototypes of included functions */
 region *duplicate_region(region *source, const char *name);
 void free_region(region *reg);
@@ -8,6 +13,7 @@ region *copy_region(region *from, region *to);
 int create_full_region(SEA_VIEW *view, const char *name);
 void save_active_region(SEA_VIEW *view, int ask_confirm);
 void hide_region_line(SEA_VIEW *view);
+int create_codon_position_mask(SEA_VIEW *view, position_mask mask, const char *regname);
 int rename_current_region(SEA_VIEW *view, const char *name);
 void delete_region(SEA_VIEW *view, int rang);
 void activate_region_line(SEA_VIEW *view, int rang);
@@ -15,6 +21,7 @@ void ajout_segment(region *maregion, int debut, int fin);
 int suppr_segment(region *maregion, int site, char *line);
 list_segments *get_segment(region *maregion, int site);
 void draw_region_line(Fl_Widget *ob, SEA_VIEW *view);
+const char *site_mask_creation_dialog(SEA_VIEW *view, position_mask *outmask);
 void regions_menu_callback(Fl_Widget *ob, void *extra);
 int begin_change_segment(SEA_VIEW *view, int new_site);
 int continue_change_segment(SEA_VIEW *view, int new_site);
@@ -118,9 +125,9 @@ while(old != NULL) {
 	}
 maregion = (region *)calloc(1,sizeof(region));
 if(maregion ==  NULL) out_of_memory();
-maregion->name=(char *)malloc(strlen(name)+1);
+maregion->name = (char *)malloc(strlen(name)+1);
 if(maregion->name ==  NULL) out_of_memory();
-strcpy(maregion->name,name);
+strcpy(maregion->name, name);
 maregion->list = (list_segments *)calloc(1, sizeof(list_segments));
 if(maregion->list ==  NULL) out_of_memory();
 maregion->list->debut = 1;
@@ -141,13 +148,73 @@ if(view->active_region == NULL) return TRUE; /*mess no mem serait mieux */
 Fl_Menu_Item *items = (Fl_Menu_Item *) ((Fl_Menu_ *)view->menu_file)->menu();
 items[SAVE_REGIONS].activate();
 view->region_length = view->seq_length;
-memset(view->region_line, 'x', view->region_length);
+memset(view->region_line, 'X', view->region_length);
 view->region_line[view->region_length] = 0;
 compute_size_params(view, TRUE);
 return TRUE;
 }
 
 
+int create_codon_position_mask(SEA_VIEW *view, position_mask mask, const char *regname)
+{
+  list_regions *old, *new_list;
+  region *maregion;
+  list_segments *segment, *previous;
+  int i;
+  /* le nom existe-t-il deja? */
+  old = view->regions;
+  while(old != NULL) {
+	if(strcmp(regname, old->element->name) == 0) return FALSE;
+	old = old->next;
+  }
+  maregion = (region *)calloc(1,sizeof(region));
+  if(maregion ==  NULL) out_of_memory();
+  maregion->name = (char *)malloc(strlen(regname + 1));
+  if(maregion->name ==  NULL) out_of_memory();
+  strcpy(maregion->name, regname);
+  
+  memset(view->region_line, '-', view->seq_length);
+  previous = NULL;
+  for(i = 1; i <= view->seq_length; i += 3) {
+	segment = (list_segments *)calloc(1, sizeof(list_segments));
+	if(mask == position_12) {
+	  segment->debut = i;
+	  segment->fin = i + 1;
+	  view->region_line[i - 1] = 'X';
+	  view->region_line[i] = 'X';
+	}
+	else {
+	  segment->debut = i + mask;
+	  segment->fin = segment->debut;
+	  view->region_line[i - 1 + mask] = 'X';
+	}
+	if(previous != NULL) previous->next = segment;
+	else maregion->list = segment;
+	previous = segment;
+	}
+  view->region_length = view->seq_length;
+  view->region_line[view->region_length] = 0;
+
+  new_list = (list_regions *)calloc(1, sizeof(list_regions));
+  if(new_list ==  NULL) out_of_memory();
+  new_list->element = maregion;
+  /* placer nouvelle region en fin de liste des regions connues */
+  if(view->regions == NULL)
+	view->regions = new_list;
+  else	{
+	old = view->regions;
+	while(old->next != NULL) old = old->next;
+	old->next = new_list;
+  }
+  view->active_region = duplicate_region(maregion, maregion->name);
+  if(view->active_region == NULL) return TRUE; /*mess no mem serait mieux */
+  Fl_Menu_Item *items = (Fl_Menu_Item *) ((Fl_Menu_ *)view->menu_file)->menu();
+  items[SAVE_REGIONS].activate();
+  compute_size_params(view, TRUE);
+  return TRUE;  
+}
+
+
 int rename_current_region(SEA_VIEW *view, const char *name)
 {
 list_regions *old, *new_list;
@@ -372,23 +439,80 @@ fl_draw(view->region_line + offset, l_line, x, y);
 }  
 
 
+const char *site_mask_creation_dialog(SEA_VIEW *view, position_mask *outmask)
+{
+  const char *retval;
+  static char custom_name[50];
+  *outmask = custom_mask;
+  Fl_Window *w = new Fl_Window(300,180);
+  w->label("Site set creation dialog");
+  Fl_Input *input = new Fl_Input(180, 20, 100, 20, "Name of custom site set?");
+  input->value("all sites");
+  Fl_Group  *g = new Fl_Group(0,25,w->w(),100);
+  Fl_Round_Button *mask[4];
+  mask[0] = new Fl_Round_Button(30, input->y() + 30, 150, 20, position_mask_names[0]);
+  mask[0]->type(FL_RADIO_BUTTON);
+  mask[1] = new Fl_Round_Button(mask[0]->x(), mask[0]->y() + 20, 150, 20, position_mask_names[1]);
+  mask[1]->type(FL_RADIO_BUTTON);
+  mask[2] = new Fl_Round_Button(mask[0]->x(), mask[1]->y() + 20, 150, 20, position_mask_names[2]);
+  mask[2]->type(FL_RADIO_BUTTON);
+  mask[3] = new Fl_Round_Button(mask[0]->x(), mask[2]->y() + 20, 180, 20, position_mask_names[3]);
+  mask[3]->type(FL_RADIO_BUTTON);
+  g->end();
+  if(view->protein) g->deactivate();
+  Fl_Return_Button *ok = new Fl_Return_Button(170, mask[3]->y() + mask[3]->h() + 10, 50, 30, "Ok");
+  Fl_Button *cancel = new Fl_Button(ok->x() + ok->w() + 5, ok->y(), 60, 30, "Cancel");
+  w->end();
+  w->show();
+  while(TRUE) {
+	Fl_Widget *o = Fl::readqueue();
+	if (!o) Fl::wait();
+	else if(o == mask[0] || o == mask[1] || o == mask[2] || o == mask[3]) {
+	  ((Fl_Round_Button*)o)->setonly();
+	  input->deactivate();
+	  if(o == mask[0]) {*outmask = position_1;}
+	  else if(o == mask[1]) {*outmask = position_2;}
+	  else if(o == mask[2]) {*outmask = position_3;}
+	  else {*outmask = position_12;}
+	  retval = position_mask_names[*outmask];
+	}
+	else if(o == cancel) {
+	  retval = NULL;
+	  break;
+	  }
+	else if(o == ok) {
+	  if(input->active()) {
+		strncpy(custom_name, input->value(), sizeof(custom_name) - 1);
+		retval = custom_name;
+		}
+	  break;
+	  }
+	}
+  delete w;
+  return retval;
+}
+
+
 void regions_menu_callback(Fl_Widget *ob, void *extra)
 {
-SEA_VIEW *view;
-const char *reg_name;
-int taille, valeur, reponse = ((Fl_Menu_ *)ob)->value() + 1;
-Fl_Menu_Item *items = (Fl_Menu_Item *) ((Fl_Menu_ *)ob)->menu();
+  SEA_VIEW *view;
+  const char *reg_name;
+  position_mask mask;
+  int taille, valeur, ok, reponse = ((Fl_Menu_ *)ob)->value() + 1;
+  Fl_Menu_Item *items = (Fl_Menu_Item *) ((Fl_Menu_ *)ob)->menu();
 
-if(reponse <= 0 ) return;
-view = (SEA_VIEW *) ((user_data_plus *)extra)->p;
-get_menu_taille_valeur(ob, &taille, &valeur);
-if(reponse == 1) { /* create new region */
-	reg_name = fl_input("New region name?", "all seqs");
+  if(reponse <= 0 ) return;
+  view = (SEA_VIEW *) ((user_data_plus *)extra)->p;
+  get_menu_taille_valeur(ob, &taille, &valeur);
+  if(reponse == 1) { /* create new region */
+	reg_name = site_mask_creation_dialog(view, &mask);
 	if(reg_name == NULL) return;
-	if( !create_full_region(view, reg_name) ) {
-		fl_alert("This name is already used: %s",reg_name);
-		return;
-		}
+	if(mask == custom_mask) ok = create_full_region(view, reg_name);
+	else ok = create_codon_position_mask(view, mask, reg_name);
+	if(!ok) {
+	  fl_alert("This name is already used: %s", reg_name);
+	  return;
+	  }
 	view->DNA_obj->redraw();
 	items[0].deactivate();
 	items[1].activate();
@@ -399,61 +523,61 @@ if(reponse == 1) { /* create new region */
 	taille++;
 	update_menu_taille_valeur(ob, taille, taille, 5);
 	}
-else if(reponse == 2) { /* save changes in current region */
-	save_active_region(view, FALSE);
-	}
-else if(reponse == 3) { /* rename current region */
-	reg_name = fl_input("New region name?", "all seqs");
-	if(reg_name == NULL) return;
-	if( !rename_current_region(view, reg_name) ) {
-		fl_alert("This name is already used: %s",reg_name);
-		return;
-		}
-	view->DNA_obj->redraw();
-	((Fl_Menu_ *)ob)->add(reg_name, 0, NULL, NULL, FL_MENU_RADIO);
-	taille++;
-	update_menu_taille_valeur(ob, taille, taille, 5);
-	}
-else if(reponse == 4) { /* hide current region */
-	hide_region_line(view);
-	view->DNA_obj->redraw();
-	items[0].activate();
-	items[1].deactivate();
-	items[2].deactivate();
-	items[3].deactivate();
-	items[4].deactivate();
-	update_menu_taille_valeur(ob, taille, 0, 5);
-	}
-else if(reponse == 5) { /* delete current region */
-	list_regions *list;
-	int tmp;
-	tmp = valeur;
-	list = view->regions;
-	if(tmp > 1) while( --tmp) list= list->next;
-	if( fl_choice("Confirm deletion of region:\n%s", "Cancel", "Delete", NULL, list->element->name) ) {
-		delete_region(view, valeur);
-		view->DNA_obj->redraw();
-		items[0].activate();
-		items[1].deactivate();
-		items[2].deactivate();
-		items[3].deactivate();
-		items[4].deactivate();
-		((Fl_Menu_ *)ob)->remove(valeur + 5 - 1);
-		init_menu_taille_valeur(ob, taille - 1, 0, 5);
-		}	
-	}
-else	{ /* choix d'une region courante */
-	valeur = reponse - 5;
-	activate_region_line(view, valeur);
-	compute_size_params(view, TRUE);
-	view->DNA_obj->redraw();
-	update_menu_taille_valeur(ob, taille, valeur, 5);
-	items[0].deactivate();
-	items[1].activate();
-	items[2].activate();
-	items[3].activate();
-	items[4].activate();
-	}
+  else if(reponse == 2) { /* save changes in current region */
+	  save_active_region(view, FALSE);
+	  }
+  else if(reponse == 3) { /* rename current region */
+	  reg_name = fl_input("New set name?", view->active_region->name);
+	  if(reg_name == NULL) return;
+	  if( !rename_current_region(view, reg_name) ) {
+		  fl_alert("This name is already used: %s",reg_name);
+		  return;
+		  }
+	  view->DNA_obj->redraw();
+	  ((Fl_Menu_ *)ob)->add(reg_name, 0, NULL, NULL, FL_MENU_RADIO);
+	  taille++;
+	  update_menu_taille_valeur(ob, taille, taille, 5);
+	  }
+  else if(reponse == 4) { /* hide current region */
+	  hide_region_line(view);
+	  view->DNA_obj->redraw();
+	  items[0].activate();
+	  items[1].deactivate();
+	  items[2].deactivate();
+	  items[3].deactivate();
+	  items[4].deactivate();
+	  update_menu_taille_valeur(ob, taille, 0, 5);
+	  }
+  else if(reponse == 5) { /* delete current region */
+	  list_regions *list;
+	  int tmp;
+	  tmp = valeur;
+	  list = view->regions;
+	  if(tmp > 1) while( --tmp) list= list->next;
+	  if( fl_choice("Confirm deletion of set:\n%s", "Cancel", "Delete", NULL, list->element->name) ) {
+		  delete_region(view, valeur);
+		  view->DNA_obj->redraw();
+		  items[0].activate();
+		  items[1].deactivate();
+		  items[2].deactivate();
+		  items[3].deactivate();
+		  items[4].deactivate();
+		  ((Fl_Menu_ *)ob)->remove(valeur + 5 - 1);
+		  init_menu_taille_valeur(ob, taille - 1, 0, 5);
+		  }	
+	  }
+  else	{ /* choix d'une region courante */
+	  valeur = reponse - 5;
+	  activate_region_line(view, valeur);
+	  compute_size_params(view, TRUE);
+	  view->DNA_obj->redraw();
+	  update_menu_taille_valeur(ob, taille, valeur, 5);
+	  items[0].deactivate();
+	  items[1].activate();
+	  items[2].activate();
+	  items[3].activate();
+	  items[4].activate();
+	  }
 }
 
 
