@@ -18,6 +18,7 @@ typedef enum {observed_pdist, Poisson_pdist, Kimura_pdist} protein_distances;
 typedef enum {observed_ndist, Jukes_Cantor, Kimura2P, HKY, LogDet, Ka, Ks} nucleotide_distances;
 
 allseq *view_to_allseq(SEA_VIEW *view, int remove_all_gaps);
+void gaps_as_unknown_states(allseq *phyml_seqs, int protein);
 void free_after_view_to_allseq(allseq *phyml_seqs);
 void bootstrap_weights(allseq *seqs, int use_codons);
 void savedistinfile(matrix *mat, const char *dirname);
@@ -307,6 +308,12 @@ void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 		for(i = 0; i < phyml_seqs->n_otu; i++) lwlseqs[i] = phyml_seqs->c_seq[i]->state;
 		}
 	matrix *phyml_mat = calc_dist_matrix(phyml_seqs, distkind, view->protein, lwlseqs, FALSE);
+	if(phyml_mat == NULL) {
+	  free_after_view_to_allseq(phyml_seqs);
+	  if( using_kaks ) free(lwlseqs);
+	  else fl_alert("Can't compute distances because sequences are too divergent.\nAre they aligned?");
+	  return;
+	  }
 	if(user_tree != NULL) {//process user tree
 		extern char *least_squares_brl(const char *tree, int nbseq, double **dist, char **names);
 		char *p, **usednames;
@@ -341,11 +348,6 @@ void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 		free_after_view_to_allseq(phyml_seqs);
 		return;
 		}
-	if(using_kaks && (phyml_mat == NULL)) {
-		free_after_view_to_allseq(phyml_seqs);
-		free(lwlseqs);
-		return;
-		}
 	phyml_mat->method = !use_bionj; //1:NJ 0:BioNJ
 	phyml_mat->tree = Make_Tree_From_Scratch(phyml_seqs->n_otu, phyml_seqs);
 	Bionj(phyml_mat);
@@ -362,7 +364,7 @@ void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 				if(*interrupted) break;
 				bootstrap_weights(phyml_seqs, using_kaks );
 				phyml_mat = calc_dist_matrix(phyml_seqs, distkind, view->protein, lwlseqs, TRUE);
-				if(using_kaks && (phyml_mat == NULL)) {
+				if(phyml_mat == NULL) {
 					total = 0;
 					break;
 					}
@@ -396,7 +398,10 @@ void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 	if(using_kaks) {
 		free(lwlseqs);
 		}
-	if(display_tree == NULL) return;
+	if(display_tree == NULL) {
+	  if( !using_kaks ) fl_alert("Can't compute distances because sequences are too divergent.\nAre they aligned?");
+	  return;
+	}
 	//add label to tree
 	char *tree = (char *)malloc(strlen(tree_label) + strlen(display_tree) + 4);
 	sprintf(tree, "[%s] %s", tree_label, display_tree);
@@ -451,7 +456,8 @@ matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs
 		phyml_mat = Obs_Dist(seqs, &jcmodel);
 	}
 	else if(distkind == Kimura_pdist ) {//Kimura protein distance
-		phyml_mat = Kimura_p_Dist(seqs);
+	  phyml_mat = Kimura_p_Dist(seqs);
+	  if(phyml_mat == NULL) return NULL;
 	}
 	else if(distkind == Poisson_pdist || distkind == Jukes_Cantor) {//JC or Poisson
 		model jcmodel;
@@ -459,15 +465,19 @@ matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs
 		jcmodel.datatype = (protein ? 1 : 0);
 		jcmodel.ns = (protein ? 20 : 4);
 		phyml_mat = JC69_Dist(seqs, &jcmodel);
+		if(phyml_mat == NULL) return NULL;
 	}
 	else if((distkind == Kimura2P) && ! protein) {//K2P
 		phyml_mat = K80_dist_nogamma(seqs);
+		if(phyml_mat == NULL) return NULL;
 	}
 	else if(distkind == HKY && ! protein) {//HKY
 		phyml_mat = HKY_dist(seqs);
+		if(phyml_mat == NULL) return NULL;
 	}
 	else if(distkind == LogDet && ! protein) {//LogDet
 		phyml_mat = LOGDET_dist(seqs);
+		if(phyml_mat == NULL) return NULL;
 	}
 	else if( (distkind == Ka || distkind == Ks) && ! protein) {//Ka or Ks
 		int sat1, sat2, count = 0, err = 0, i, j;
@@ -699,10 +709,11 @@ void phyml_dialog(SEA_VIEW *view)
 	for(int i = view->offset_to_local_trees; i < count; i++) u_choice->add( (items + i)->label() );
 	u_choice->value(0);
 	if(count <= view->offset_to_local_trees) {u_menutree->deactivate(); u_choice->deactivate(); }
-	u_random = new Fl_Round_Button(3, u_menutree->y() + u_menutree->h() + 5, 80, 20, "Random");
+	u_random = new Fl_Round_Button(3, u_menutree->y() + u_menutree->h() + 5, 80, 20, "Add: ");
 	u_random->deactivate();
 	u_random->type(FL_RADIO_BUTTON);
-	u_random_count = new Fl_Int_Input(u_random->x() + u_random->w() + 130, u_random->y(), 35, 20, "# of random starts:");
+	u_random_count = new Fl_Int_Input(u_random->x() + u_random->w(), u_random->y(), 35, 20, "random starts");
+	u_random_count->align(FL_ALIGN_RIGHT);
 	u_random_count->value("5");
 	u_group->end();
 	y = u_group->y() + u_group->h() + 5;
@@ -1006,6 +1017,16 @@ allseq *view_to_allseq(SEA_VIEW *view, int remove_all_gaps)
 }
 
 
+void gaps_as_unknown_states(allseq *phyml_seqs, int protein)
+{
+  char unknown = (protein ? 'X' : 'N');
+  for( int i = 0; i < phyml_seqs->n_otu; i++) {
+	for( int j = 0; j < phyml_seqs->clean_len; j++) {
+	  if(phyml_seqs->c_seq[i]->state[j] == '-') phyml_seqs->c_seq[i]->state[j] = unknown;
+	  }
+	}
+}
+
 void free_after_view_to_allseq(allseq *phyml_seqs)
 {
 	int i;
@@ -1121,9 +1142,14 @@ matrix *Kimura_p_Dist(allseq *data)
 	matrix *mat = Obs_Dist(data, &jcmodel);
 	for(i = 0; i < data->n_otu; i++) {
 		for(j = i + 1; j < data->n_otu; j++) {
-			mat->dist[i][j] = -log(1 - mat->dist[i][j] - 0.2 * mat->dist[i][j] * mat->dist[i][j]);
-			mat->dist[j][i] = mat->dist[i][j];
+		  phydbl x = 1 - mat->dist[i][j] - 0.2 * mat->dist[i][j] * mat->dist[i][j];
+		  if(x <= 0) {
+			Free_Mat(mat);
+			return NULL;
 			}
+		  mat->dist[i][j] = -log(x);
+		  mat->dist[j][i] = mat->dist[i][j];
+		  }
 		}
 	return mat;
 }
@@ -1266,9 +1292,9 @@ matrix *K80_dist_nogamma(allseq *data)
 		
 		if((1-2*mat->P[i][j]-mat->Q[i][j] <= .0) || (1-2*mat->Q[i][j] <= .0))
 		{
-			mat->dist[i][j] = -1.;
-			mat->dist[j][i] = -1.;
-			continue;
+		  Free_Mat(mat);
+		  mat = NULL;
+		  goto out;
 		}
 		
 /*		mat->dist[i][j] = (g_shape/2)*
@@ -1276,35 +1302,36 @@ matrix *K80_dist_nogamma(allseq *data)
 		 0.5*pow(1-2*mat->Q[i][j],-1./g_shape) - 1.5); */
 		
 		mat->dist[i][j] = -0.5 * log(1-2*mat->P[i][j]-mat->Q[i][j]) - 0.25 * log(1-2*mat->Q[i][j]);
-		
-		if(mat->dist[i][j] > DIST_MAX)
-		{
-			mat->dist[i][j] = DIST_MAX;
-		}
 		mat->dist[j][i] = mat->dist[i][j];
 	}
 	
-	For(i,data->n_otu) free(len[i]);
+out:For(i,data->n_otu) free(len[i]);
 	free(len);
 	return mat;
 }
 
 matrix *HKY_dist(allseq *data)
 {
-	int i,j;
-	matrix *mat;
-			
-	mat = Make_Mat(data->n_otu);
-	Init_Mat(mat,data);
-	For(i,data->n_otu - 1)
-    {
-		for(j = i + 1; j < data->n_otu; j++)
-		{
-			mat->dist[i][j] = hky(data->c_seq[i]->state, data->c_seq[j]->state, data->clean_len, data->wght);
-			mat->dist[j][i] = mat->dist[i][j];
-		}
-	}
-	return mat;
+  int i,j;
+  matrix *mat;
+  double x;
+		  
+  mat = Make_Mat(data->n_otu);
+  Init_Mat(mat,data);
+  For(i,data->n_otu - 1)
+  {
+	  for(j = i + 1; j < data->n_otu; j++)
+	  {
+		x = hky(data->c_seq[i]->state, data->c_seq[j]->state, data->clean_len, data->wght);
+		if(x == -1) {
+		  Free_Mat(mat);
+		  return NULL;
+		  }
+		mat->dist[i][j] = x;
+		mat->dist[j][i] = mat->dist[i][j];
+	  }
+  }
+  return mat;
 }
 			
 
@@ -1484,7 +1511,7 @@ void parsimony_dialog(SEA_VIEW *view)
 		nreplicates, replicates_done, user_tree, nogaps; 
 	char *tree, *final_tree;
 	const char *p, *progname;
-	Fl_Window *w = new Fl_Window(255, 200);
+	Fl_Window *w = new Fl_Window(255, 225);
 	w->set_modal();
 	w->label("Parsimony analysis");
 	Fl_Check_Button *w_dojumble = new Fl_Check_Button(2, 5, 170, 20, "Randomize seq. order");
@@ -1494,7 +1521,11 @@ void parsimony_dialog(SEA_VIEW *view)
 	w_jumbles->value("5");
 	Fl_Check_Button *w_nogaps = new Fl_Check_Button(2, w_dojumble->y() + w_dojumble->h() + 5, 160, 20, "Ignore all gap sites");
 	w_nogaps->value(1);
-	Fl_Int_Input *w_best = new Fl_Int_Input(195, w_nogaps->y() + w_nogaps->h() + 5, 50, 20, "Equally best trees retained");
+	Fl_Check_Button *w_gapsunknown = new Fl_Check_Button(2, w_nogaps->y() + w_nogaps->h() + 5, 180, 20, 
+														 "Gaps as unknown states");
+	w_gapsunknown->value(0);
+	w_gapsunknown->deactivate();
+	Fl_Int_Input *w_best = new Fl_Int_Input(195, w_gapsunknown->y() + w_gapsunknown->h() + 5, 50, 20, "Equally best trees retained");
 	w_best->align(FL_ALIGN_LEFT);
 	w_best->value("10000");
 	Fl_Round_Button *w_dobootstrap = new Fl_Round_Button(2, w_best->y() + w_best->h() + 5,
@@ -1523,7 +1554,13 @@ void parsimony_dialog(SEA_VIEW *view)
 	w->callback(interrupt_callback, &interrupted);
 	w->size(w->w(), w_ok->y() + w_ok->h() + 5);
 	w->show();
-	while(!started && !interrupted) Fl::wait();
+	while(!started && !interrupted) {
+	  Fl::wait();
+	  if(w_gapsunknown->value()) { w_nogaps->value(0); w_nogaps->deactivate(); }
+	  else w_nogaps->activate();
+	  if(w_nogaps->value()) { w_gapsunknown->value(0); w_gapsunknown->deactivate(); }
+	  else w_gapsunknown->activate();
+	  }
 	if(interrupted) {
 		delete w;
 		return;
@@ -1539,6 +1576,10 @@ void parsimony_dialog(SEA_VIEW *view)
 	use_bootstrap = w_dobootstrap->value();
 	user_tree = w_user->value();
 	nogaps = w_nogaps->value();
+	char gapsunknown[25];
+	if(w_gapsunknown->value())
+	sprintf(gapsunknown, ", gaps treated as %c", (view->protein ? 'X' : 'N') );
+	else gapsunknown[0] = 0;
 	if(use_bootstrap) {
 		p = w_btrepl->value();
 		sscanf(p, "%d", &nreplicates);
@@ -1552,6 +1593,7 @@ void parsimony_dialog(SEA_VIEW *view)
 
 	my_watch_cursor(view->dnawin);
 	allseq *tmpallseqs = view_to_allseq(view, nogaps);
+	if(w_gapsunknown->value()) gaps_as_unknown_states(tmpallseqs, view->protein);
 	char **pars_seqs = (char **)malloc(tmpallseqs->n_otu * sizeof(char *));
 	char **pars_names = (char **)malloc(tmpallseqs->n_otu * sizeof(char *));
 	for( i = 0; i < tmpallseqs->n_otu; i++) {
@@ -1565,8 +1607,8 @@ void parsimony_dialog(SEA_VIEW *view)
 				view->protein, &steps, maxbest, nreplicates, w_count, &interrupted, &replicates_done);
 		delete w;
 		final_tree = (char *)malloc(strlen(tree) + 150);
-		sprintf(final_tree, "[%s, bootstrap with %d replic., %d steps, %d sites (%d informative)]%s",
-				progname, replicates_done, steps, tmpallseqs->clean_len, info, tree);
+		sprintf(final_tree, "[%s, bootstrap with %d replic., %d steps, %d sites (%d informative)%s]%s",
+				progname, replicates_done, steps, tmpallseqs->clean_len, info, gapsunknown, tree);
 		}
 	else if(user_tree) {
 		char *differs, *input_tree;
@@ -1598,16 +1640,16 @@ void parsimony_dialog(SEA_VIEW *view)
 			}
 		else {
 			final_tree = (char *)malloc(strlen(tree) + 150);
-			sprintf(final_tree, "[%s, user-tree, %d steps, %d sites (%d informative)]%s",
-					progname, steps, tmpallseqs->clean_len, info, tree);
+			sprintf(final_tree, "[%s, user-tree, %d steps, %d sites (%d informative)%s]%s",
+					progname, steps, tmpallseqs->clean_len, info, gapsunknown, tree);
 			}
 	}
 	else {
 		tree = parsimony_with_jumble(pars_seqs, pars_names, tmpallseqs->n_otu, njumbles, view->protein, 
 								 &steps, &count, maxbest);
 		final_tree = (char *)malloc(strlen(tree) + 150);
-		sprintf(final_tree, "[%s, %d best trees%s, %d steps, %d sites (%d informative)]%s",
-			progname, count, count > 1 ? " (strict consensus)" : "", steps, tmpallseqs->clean_len, info, tree);
+		sprintf(final_tree, "[%s, %d best trees%s, %d steps, %d sites (%d informative)%s]%s",
+			progname, count, count > 1 ? " (strict consensus)" : "", steps, tmpallseqs->clean_len, info, gapsunknown, tree);
 		}
 	if(tree != NULL) free(tree);
 	free_after_view_to_allseq(tmpallseqs);
@@ -1864,8 +1906,13 @@ matrix *LOGDET_dist(allseq *data)
     {
 		for(j = i + 1; j < data->n_otu; j++)
 		{
-			mat->dist[i][j] = logdet(data->c_seq[i]->state, data->c_seq[j]->state, data->clean_len, data->wght);
-			mat->dist[j][i] = mat->dist[i][j];
+		  double x = logdet(data->c_seq[i]->state, data->c_seq[j]->state, data->clean_len, data->wght);
+		  if(x == -1.) {
+			Free_Mat(mat);
+			return NULL;
+			}
+		  mat->dist[i][j] = x;
+		  mat->dist[j][i] = x;
 		}
 	}
 	return mat;
