@@ -69,6 +69,8 @@ int chg_phys(FD_unrooted *ob, hits_field *hits );
 void log_to_phys(FD_unrooted *data, cp_point *log_pos, cp_point *phys_pos);
 double length_log_phys(FD_unrooted *data, double p);
 double length_phys_log(FD_unrooted *data, double p);
+void squared2circular(FD_nj_plot *fd_nj_plot, Fl_Choice *o);
+void circular2squared(FD_nj_plot *fd_nj_plot);
 #ifdef MICRO
 void print_unrooted(FD_unrooted *fd_unrooted);
 #endif
@@ -90,6 +92,10 @@ extern double arrondi_echelle(double x);
 extern void zoom_callback(Fl_Widget *wgt, void *data);
 extern void scroller_callback(Fl_Widget *wgt, void *data);
 extern void draw_rotated_string(const char *txt, int x, int y, float degrees /*clockwise*/);
+extern int calc_brl_for_lengthless(struct noeud *centre, struct noeud *pere);
+extern const char *preptree(FD_nj_plot *fd_nj_plot);
+extern void free_tree(FD_nj_plot *fd_nj_plot);
+extern char *ecrit_arbre_parenth(FD_nj_plot *fd_nj_plot, struct noeud *root);
 #if defined(MICRO)
 extern int MG_StartPrinting (int pagecount, int *frompage, int *topage);
 extern int MG_StartPage (void);
@@ -106,104 +112,168 @@ extern paperformat printout_pageformat;
 
 void rooted_unrooted_callback(Fl_Widget *o, void *data)
 {
-	FD_nj_plot *fd_nj_plot = (FD_nj_plot *)data;
-	Fl_Menu_Item *item = (Fl_Menu_Item *)fd_nj_plot->menu_file->menu();
-	Fl_Menu_Item *edit_item = (Fl_Menu_Item *)fd_nj_plot->menu_edit->menu();
-	if(((Fl_Choice *)o)->value() == 1) {
-		Fl_Widget *panel = fd_nj_plot->panel; //switch to circular display
-		FD_unrooted *fd_unrooted;
-		char *tree = ecrit_arbre_parenth_unrooted(fd_nj_plot, fd_nj_plot->racine);
-		if(tree == NULL) { ((Fl_Choice *)o)->value(0); return; }
-		fd_unrooted = (FD_unrooted *) calloc(1, sizeof(FD_unrooted));
-		int err = debut_arbre(tree, fd_unrooted);
-		free(tree);
-		if(err) {
-			((Fl_Choice *)o)->value(0);
-			free(fd_unrooted);
-			return;
-		}
-		panel->hide();
-		fd_nj_plot->scroller->hide();
-		fd_nj_plot->l_button->deactivate();
-		fd_nj_plot->bt_button->deactivate();
-		fd_nj_plot->full->deactivate();
-		fd_nj_plot->swap_button->deactivate();
-		fd_nj_plot->subtree->deactivate();
-		fd_nj_plot->new_outgroup->deactivate();
-		edit_item[4].deactivate();//edit header
-		edit_item[5].deactivate();//bootstrap threshold
-		edit_item[9].deactivate();//root at center
-		edit_item[fd_nj_plot->edit_shape_rank].deactivate();//edit shape
-		item[2].deactivate();//save rooted tree
-		item[3].deactivate();//save unrooted tree
-		item[8].deactivate();//Page count
-		fd_unrooted->zoomvalue = 1;
-		fd_nj_plot->zoom->value(1);
-		Fl_Window *w2 = (Fl_Window *)panel->parent();
-		Fl_Group::current(w2);
-		fd_unrooted->vert_scroller = new Fl_Scrollbar(w2->w() - 15, 0, 15, fd_nj_plot->scroller->h());
-		fd_unrooted->vert_scroller->hide();
-		fd_unrooted->vert_scroller->bounds(0, 1);
-		fd_unrooted->vert_scroller->slider_size(1);
-		((Fl_Slider *)fd_unrooted->vert_scroller)->value(0);
-		fd_unrooted->y = 0;
-		fd_unrooted->y_offset = 0;
-		fd_unrooted->vert_scroller->callback(unrooted_scroller_cb, fd_unrooted);		
-		fd_nj_plot->zoom->callback(unrooted_zoom_cb, fd_unrooted);
-		fd_unrooted->unrooted_plot = new unrooted_panel(0, 0, w2->w(), w2->h());
-		Fl_Box *box = new Fl_Box(0, 0, w2->w() - 15, w2->h() - 15);
-		box->hide();
-		w2->resizable(box);
-		fd_unrooted->unrooted_plot->color(FL_WHITE);
-		fd_unrooted->unrooted_plot->user_data(fd_unrooted);
-		fd_unrooted->hor_scroller = new Fl_Scrollbar(0, w2->h() - 15, w2->w() - 15, 15);
-		fd_unrooted->hor_scroller->hide();
-		fd_unrooted->hor_scroller->type(FL_HORIZONTAL);
-		fd_unrooted->hor_scroller->bounds(0, 1);
-		fd_unrooted->hor_scroller->slider_size(1);
-		((Fl_Slider *)fd_unrooted->hor_scroller)->value(0);
-		fd_unrooted->x = 0;
-		fd_unrooted->x_offset = 0;
-		fd_unrooted->hor_scroller->callback(unrooted_scroller_cb, fd_unrooted);
-		fd_unrooted->current_font = fd_nj_plot->font_family;
-		fd_unrooted->font_size = fd_nj_plot->font_size;
-		fd_nj_plot->fd_unrooted = (void *)fd_unrooted;
-		}
-	else {
-		FD_unrooted *fd_unrooted = (FD_unrooted *)fd_nj_plot->fd_unrooted; //switch to squared display
-		fd_unrooted->unrooted_plot->hide();
-		fd_unrooted->vert_scroller->hide();
-		fd_unrooted->hor_scroller->hide();
-//dont't call delete but Fl::delete_widget after having removed it from its group!!!
-		fd_unrooted->unrooted_plot->window()->remove( *fd_unrooted->vert_scroller );
-		fd_unrooted->unrooted_plot->window()->remove( *fd_unrooted->hor_scroller );
-		fd_unrooted->unrooted_plot->window()->remove( *fd_unrooted->unrooted_plot );
-		Fl::delete_widget(fd_unrooted->unrooted_plot);
-		Fl::delete_widget(fd_unrooted->vert_scroller);
-		Fl::delete_widget(fd_unrooted->hor_scroller);
-		free_unrooted(fd_unrooted);
-		fd_nj_plot->fd_unrooted = NULL;
-		fd_nj_plot->need_runtree = TRUE;
-		fd_nj_plot->panel->show();
-		if(fd_nj_plot->has_br_length) fd_nj_plot->l_button->activate();
-		if(fd_nj_plot->has_internal) fd_nj_plot->bt_button->activate();
-		if(fd_nj_plot->zoomvalue > 1.1) fd_nj_plot->scroller->show();
-		fd_nj_plot->zoom->value(fd_nj_plot->zoomvalue);
-		fd_nj_plot->zoom->callback(zoom_callback, fd_nj_plot);
-		fd_nj_plot->full->activate();
-		fd_nj_plot->swap_button->activate();
-		fd_nj_plot->subtree->activate();
-		fd_nj_plot->new_outgroup->activate();
-		edit_item[4].activate();//edit header
-		edit_item[5].activate();
-		edit_item[9].activate();//root at center
-		edit_item[fd_nj_plot->edit_shape_rank].activate();//edit shape
-		item[2].activate();
-		item[3].activate();
-		item[8].activate();
+  FD_nj_plot *fd_nj_plot = (FD_nj_plot *)data;
+  char *keep, *keepn, *keepl;
+  if(((Fl_Choice *)o)->value() == 0) {//goto squared display
+	if(fd_nj_plot->fd_unrooted != NULL) circular2squared(fd_nj_plot);
+	if(fd_nj_plot->lost_br_lengths) {
+	  fd_nj_plot->lost_br_lengths = FALSE;
+	  keep = fd_nj_plot->current_tree;
+	  keepn = fd_nj_plot->tree_name;
+	  keepl = fd_nj_plot->tree_label;
+	  fd_nj_plot->current_tree = NULL;
+	  fd_nj_plot->tree_name = NULL;
+	  fd_nj_plot->tree_label = NULL;
+	  free_tree(fd_nj_plot);
+	  fd_nj_plot->current_tree = keep;
+	  fd_nj_plot->tree_name = keepn;
+	  fd_nj_plot->tree_label = keepl;
+	  preptree(fd_nj_plot);//reinterpret tree for its branch lengths
+	}
+	if(fd_nj_plot->has_br_length) fd_nj_plot->l_button->activate();
+	fd_nj_plot->new_outgroup->activate();
+	fd_nj_plot->swap_button->activate();
+	fd_nj_plot->need_runtree = TRUE;
+	fd_nj_plot->panel->show();
+	fd_nj_plot->panel->window()->redraw();
+  }
+  else if(((Fl_Choice *)o)->value() == 1) {//goto circular display
+	if(fd_nj_plot->lost_br_lengths) {
+	  fd_nj_plot->lost_br_lengths = FALSE;
+	  keep = fd_nj_plot->current_tree;
+	  keepn = fd_nj_plot->tree_name;
+	  keepl = fd_nj_plot->tree_label;
+	  fd_nj_plot->current_tree = NULL;
+	  fd_nj_plot->tree_name = NULL;
+	  fd_nj_plot->tree_label = NULL;
+	  free_tree(fd_nj_plot);
+	  fd_nj_plot->current_tree = keep;
+	  fd_nj_plot->tree_name = keepn;
+	  fd_nj_plot->tree_label = keepl;
+	  preptree(fd_nj_plot);//reinterpret tree for its branch lengths
+	  for(int i=0; i<fd_nj_plot->notu; i++) {
+		strcpy(fd_nj_plot->tabtax[i]->nom, fd_nj_plot->labels[i]);
+	  }
+	}
+	if(fd_nj_plot->fd_unrooted == NULL) squared2circular(fd_nj_plot, (Fl_Choice *)o);
+	}
+  else if(((Fl_Choice *)o)->value() == 2) {//goto Cladogram display
+	if(fd_nj_plot->fd_unrooted != NULL) circular2squared(fd_nj_plot);
+	keep = ecrit_arbre_parenth(fd_nj_plot, fd_nj_plot->racine);
+	free(fd_nj_plot->current_tree);
+	fd_nj_plot->current_tree = keep;
+	fd_nj_plot->lost_br_lengths = TRUE;
+	fd_nj_plot->has_br_length = FALSE;
+	calc_brl_for_lengthless(fd_nj_plot->racine, NULL);
+	fd_nj_plot->need_runtree = TRUE;
+	fd_nj_plot->l_button->deactivate();
+	fd_nj_plot->new_outgroup->deactivate();
+	fd_nj_plot->swap_button->deactivate();
+	fd_nj_plot->choix = show_tree;
+	fd_nj_plot->full->setonly();
+	fd_nj_plot->panel->show();
+	fd_nj_plot->panel->window()->redraw();
 	}
 }
 
+void squared2circular(FD_nj_plot *fd_nj_plot, Fl_Choice *o)
+{
+	Fl_Menu_Item *item = (Fl_Menu_Item *)fd_nj_plot->menu_file->menu();
+	Fl_Menu_Item *edit_item = (Fl_Menu_Item *)fd_nj_plot->menu_edit->menu();
+	Fl_Widget *panel = fd_nj_plot->panel;
+	FD_unrooted *fd_unrooted;
+	char *tree = ecrit_arbre_parenth_unrooted(fd_nj_plot, fd_nj_plot->racine);
+	if(tree == NULL) { o->value(0); return; }
+	fd_unrooted = (FD_unrooted *) calloc(1, sizeof(FD_unrooted));
+	int err = debut_arbre(tree, fd_unrooted);
+	free(tree);
+	if(err) {
+	  o->value(0);
+	  free(fd_unrooted);
+	  return;
+	}
+	panel->hide();
+	fd_nj_plot->scroller->hide();
+	fd_nj_plot->l_button->deactivate();
+	fd_nj_plot->bt_button->deactivate();
+	fd_nj_plot->full->deactivate();
+	fd_nj_plot->swap_button->deactivate();
+	fd_nj_plot->subtree->deactivate();
+	fd_nj_plot->new_outgroup->deactivate();
+	edit_item[4].deactivate();//edit header
+	edit_item[5].deactivate();//bootstrap threshold
+	edit_item[9].deactivate();//root at center
+	edit_item[fd_nj_plot->edit_shape_rank].deactivate();//edit shape
+	item[2].deactivate();//save rooted tree
+	item[3].deactivate();//save unrooted tree
+	item[8].deactivate();//Page count
+	fd_unrooted->zoomvalue = 1;
+	fd_nj_plot->zoom->value(1);
+	Fl_Window *w2 = (Fl_Window *)panel->parent();
+	Fl_Group::current(w2);
+	fd_unrooted->vert_scroller = new Fl_Scrollbar(w2->w() - 15, 0, 15, fd_nj_plot->scroller->h());
+	fd_unrooted->vert_scroller->hide();
+	fd_unrooted->vert_scroller->bounds(0, 1);
+	fd_unrooted->vert_scroller->slider_size(1);
+	((Fl_Slider *)fd_unrooted->vert_scroller)->value(0);
+	fd_unrooted->y = 0;
+	fd_unrooted->y_offset = 0;
+	fd_unrooted->vert_scroller->callback(unrooted_scroller_cb, fd_unrooted);		
+	fd_nj_plot->zoom->callback(unrooted_zoom_cb, fd_unrooted);
+	fd_unrooted->unrooted_plot = new unrooted_panel(0, 0, w2->w(), w2->h());
+	Fl_Box *box = new Fl_Box(0, 0, w2->w() - 15, w2->h() - 15);
+	box->hide();
+	w2->resizable(box);
+	fd_unrooted->unrooted_plot->color(FL_WHITE);
+	fd_unrooted->unrooted_plot->user_data(fd_unrooted);
+	fd_unrooted->hor_scroller = new Fl_Scrollbar(0, w2->h() - 15, w2->w() - 15, 15);
+	fd_unrooted->hor_scroller->hide();
+	fd_unrooted->hor_scroller->type(FL_HORIZONTAL);
+	fd_unrooted->hor_scroller->bounds(0, 1);
+	fd_unrooted->hor_scroller->slider_size(1);
+	((Fl_Slider *)fd_unrooted->hor_scroller)->value(0);
+	fd_unrooted->x = 0;
+	fd_unrooted->x_offset = 0;
+	fd_unrooted->hor_scroller->callback(unrooted_scroller_cb, fd_unrooted);
+	fd_unrooted->current_font = fd_nj_plot->font_family;
+	fd_unrooted->font_size = fd_nj_plot->font_size;
+	fd_nj_plot->fd_unrooted = (void *)fd_unrooted;
+}
+
+void circular2squared(FD_nj_plot *fd_nj_plot)
+{
+  Fl_Menu_Item *item = (Fl_Menu_Item *)fd_nj_plot->menu_file->menu();
+  Fl_Menu_Item *edit_item = (Fl_Menu_Item *)fd_nj_plot->menu_edit->menu();
+  FD_unrooted *fd_unrooted = (FD_unrooted *)fd_nj_plot->fd_unrooted;
+  fd_unrooted->unrooted_plot->hide();
+  fd_unrooted->vert_scroller->hide();
+  fd_unrooted->hor_scroller->hide();
+  //dont't call delete but Fl::delete_widget after having removed it from its group!!!
+  fd_unrooted->unrooted_plot->window()->remove( *fd_unrooted->vert_scroller );
+  fd_unrooted->unrooted_plot->window()->remove( *fd_unrooted->hor_scroller );
+  fd_unrooted->unrooted_plot->window()->remove( *fd_unrooted->unrooted_plot );
+  Fl::delete_widget(fd_unrooted->unrooted_plot);
+  Fl::delete_widget(fd_unrooted->vert_scroller);
+  Fl::delete_widget(fd_unrooted->hor_scroller);
+  free_unrooted(fd_unrooted);
+  fd_nj_plot->fd_unrooted = NULL;
+  if(fd_nj_plot->has_br_length) fd_nj_plot->l_button->activate();
+  if(fd_nj_plot->has_internal) fd_nj_plot->bt_button->activate();
+  if(fd_nj_plot->zoomvalue > 1.1) fd_nj_plot->scroller->show();
+  fd_nj_plot->zoom->value(fd_nj_plot->zoomvalue);
+  fd_nj_plot->zoom->callback(zoom_callback, fd_nj_plot);
+  fd_nj_plot->full->activate();
+  fd_nj_plot->swap_button->activate();
+  fd_nj_plot->subtree->activate();
+  fd_nj_plot->new_outgroup->activate();
+  edit_item[4].activate();//edit header
+  edit_item[5].activate();
+  edit_item[9].activate();//root at center
+  edit_item[fd_nj_plot->edit_shape_rank].activate();//edit shape
+  item[2].activate();
+  item[3].activate();
+  item[8].activate();
+}
 
 void unrooted_scroller_cb(Fl_Widget *wgt, void *data)
 {
