@@ -10,7 +10,7 @@ int read_mase_seqs_header(const char *masefname, char ***pseq, char ***pseqname,
 		char ***pcomments, char **pheader, char **err_message);
 int one_more_seq_found(int count1, char ***pseq, char ***pseqname, char ***pcomments);
 int read_fasta_align(const char *fname, char ***pseq, char ***pseqname, 
-		char ***pcomments, char **pheader, char **err_message);
+		char ***pcomments, char **pheader, char **err_message, int spaces_in_names);
 int read_phylip_align(const char *fname, char ***pseq, char ***pseqname, 
 		char ***pcomments, char **pheader, char **err_message);
 int read_clustal_align(const char *fname, char ***pseq, char ***pseqname, 
@@ -20,7 +20,7 @@ int read_msf_align(const char *fname, char ***pseq, char ***pseqname,
 int is_a_protein_seq(char *seq);
 int save_fasta_file(const char *fname, char **seq, char **comments,
 	char **seqname, int totseqs, int *eachlength, region *region_used,
-	int *sel_seqs, int tot_sel_seqs);
+	int *sel_seqs, int tot_sel_seqs, int spaces_in_names);
 int save_phylip_file(const char *fname, char **seq,
 	char **seqname, int totseqs, int *eachlength, region *region_used, 
 	int *sel_seqs, int tot_sel_seqs, int phylipwidnames);
@@ -48,7 +48,7 @@ char *save_alignment_or_region(const char *fname, char **seq, char **comments,
 	int numb_species_sets, int **list_species_sets, 
 	char **name_species_sets, int *sel_seqs, int tot_sel_seqs, int protein,
 	int tot_comment_lines, char **comment_name, char **comment_line, int phylipwidnames, 
-	int tot_trees, char **trees, const Fl_Menu_Item *menu_tree_items);
+	int tot_trees, char **trees, const Fl_Menu_Item *menu_tree_items, int spaces_in_fasta_names);
 int prepare_printout(const char *filename, char **seq, int totseqs,  
 	char **seqname,
 	int *eachlength, int char_per_line,
@@ -309,14 +309,15 @@ else {
 return count1;
 }
 
+
 int read_fasta_align(const char *fname, char ***pseq, char ***pseqname, 
-		char ***pcomments, char **pheader, char **err_message)
+		char ***pcomments, char **pheader, char **err_message, int spaces_in_names)
 {
 FILE *in;
 int totseqs, lseq, l2, l, lenseqs;
-char line[300], *p, *i, *provseq = NULL, c, *q;
+char line[300], *p, *i, c, *q;
 static char ret_message[200];
-char **seq, **seqname, **comments;
+char **seq, **seqname, **comments, *tmpseq = NULL;
 
 *ret_message = 0;
 *err_message = ret_message;
@@ -340,6 +341,8 @@ if(seqname == NULL) goto nomem;
 *pseq = seq; *pcomments = comments; *pseqname = seqname;
 
 lenseqs = MAXLENSEQ;
+tmpseq = (char *)malloc(lenseqs + 1);
+if(tmpseq == NULL) goto nomem;
 totseqs = -1;
 i = fgets(line, sizeof(line), in);
 if(line[0] != '>') {
@@ -353,7 +356,14 @@ while( i != NULL ){
 	q = line + strlen(line) - 1;
 	while(q > line + 1 && (*q == '\n' || *q == '\r')) *(q--) = 0;
 	totseqs++;
-	p = line + 1; while(*p != ' ' && *p != '\n') p++;
+	p = line + 1; 
+	if(spaces_in_names) {
+	  while(*p != '\n') p++;
+	  while(*(p-1) == ' ') p--;
+	  }
+	else {
+	  while(*p != ' ' && *p != '\n') p++;
+	  }
 	l = p - line - 1;
 	if( (seqname[totseqs] = (char *)malloc(l+1)) == NULL)goto nomem;
 	memcpy(seqname[totseqs], line + 1, l); seqname[totseqs][l] = 0;
@@ -369,38 +379,33 @@ while( i != NULL ){
 			}
 		}
 	else comments[totseqs] = NULL;
-	seq[totseqs] = (char *)malloc(lenseqs+1);
-	if(seq[totseqs] == NULL) goto nomem;
 	lseq = 0;
 	while( (i=fgets(line, sizeof(line), in))!= NULL && *i != '>' ) {
 		l2 = strlen(line);
 		if( line[l2 - 1] == '\n' ) l2--;
 	   	while(l2>0 && line[l2-1]==' ')l2--;
 		if(lseq + l2 > lenseqs) {
-			char *temp;
 			lenseqs += MAXLENSEQ;
-			temp = (char *)malloc(lenseqs+1);
-			if(temp == NULL) goto nomem;
-			memcpy(temp, seq[totseqs], lseq);
-			free(seq[totseqs]);
-			seq[totseqs] = temp;
+			tmpseq= (char *)realloc(tmpseq, lenseqs + 1);
+			if(tmpseq == NULL) goto nomem;
 			}
 /* copy seq data excluding spaces (because of gblocks) */
-		p = seq[totseqs]+lseq;
+		p = tmpseq+lseq;
 		q = line;
 		while (q < line + l2) {
 			if(*q != ' ') *(p++) = *q;
 			q++;
 			}
-		lseq += p - (seq[totseqs]+lseq);
+		lseq += p - (tmpseq+lseq);
 		}
-	seq[totseqs][lseq]='\0';
-/* convert all to upper case 
-	p = seq[totseqs] - 1; while( *(++p) != 0 ) *p = toupper(*p); */
+	tmpseq[lseq]='\0';
+	seq[totseqs] = (char *)malloc(lseq + 1);
+	if(seq[totseqs] == NULL) goto nomem;
+	memcpy(seq[totseqs], tmpseq, lseq + 1);
 	}
 fini:
 fclose(in);
-if(provseq != NULL) free(provseq);
+if(tmpseq != NULL) free(tmpseq);
 *pheader = NULL;
 return totseqs+1;
 nomem:
@@ -414,8 +419,8 @@ int read_phylip_align(const char *fname, char ***pseq, char ***pseqname,
 		char ***pcomments, char **pheader, char **err_message)
 {
 FILE *in;
-char *p, *q;
-static char line[30000];
+char *p, *q, c;
+static char line[300];
 char **seq, **comments, **seqname;
 int totseqs, lenseqs, i, l;
 static char ret_message[200];
@@ -443,30 +448,51 @@ for(i=0; i<totseqs; i++) {
 	comments[i] = NULL;
 	}
 for(i=0; i<totseqs; i++) {
-	fgets(line,sizeof(line),in);
-	p = strstr(line, " "); if(p == NULL) p = line + 10;
-	if( (seqname[i] = (char *)malloc(p - line + 1) ) == NULL ) goto nomem;
-	memcpy(seqname[i], line, p - line); seqname[i][p - line] = 0;
-	q = seq[i];
-	while(*p != '\n') {
-		if(*p != ' ') *(q++) = *p;
-		p++;
+  fgets(line,sizeof(line),in);
+  p = strstr(line, " "); if(p == NULL) p = line + 10;
+  if( (seqname[i] = (char *)malloc(p - line + 1) ) == NULL ) goto nomem;
+  memcpy(seqname[i], line, p - line); seqname[i][p - line] = 0;
+  q = seq[i];
+  while(*p != 0 && *p != '\n') {
+	  if(*p != ' ') {
+		if(q - seq[i] >= lenseqs) goto badfile;
+		*(q++) = *p;
 		}
+	  p++;
+	  }
+  c = *p;
+  if(c == '\n') continue;
+  while(TRUE) {
+	c = fgetc(in);
+	if(c == EOF) goto badfile;
+	if(c == '\n' || c == '\r') break;
+	if(c != ' ') {
+	  if(q - seq[i] >= lenseqs) goto badfile;
+	  *(q++) = c;
+	  }
 	}
+  if(c == '\r') {c = fgetc(in); if(c != '\n') ungetc(c, in); }
+  }
 l = q - seq[totseqs - 1];
 while( l < lenseqs) {
-	if(fgets(line,sizeof(line),in) == NULL) goto badfile;
-	for(i=0; i<totseqs; i++) {
-		p = fgets(line,sizeof(line),in);
-		if(p == NULL) goto badfile;
-		q = seq[i] + l;
-		while(*p != '\n' && q - seq[i] < lenseqs) {
-			if(*p != ' ') *(q++) = *p;
-			p++;
-			}
+  do c = fgetc(in); while(c != '\n' && c != '\r' && c != EOF);
+  if(c == EOF) goto badfile;
+  if(c == '\r') {c = fgetc(in); if(c != '\n') ungetc(c, in); }
+  for(i=0; i<totseqs; i++) {
+	q = seq[i] + l;
+	while(TRUE) {
+	  c = fgetc(in);
+	  if(c == EOF) goto badfile;
+	  if(c == '\n' || c == '\r') break;
+	  if(c != ' ') {
+		if(q - seq[i] >= lenseqs) goto badfile;
+		*(q++) = c;
 		}
-	l = q - seq[totseqs - 1];
+	  }
+	if(c == '\r') {c = fgetc(in); if(c != '\n') ungetc(c, in); }
 	}
+  l = q - seq[totseqs - 1];
+  }
 for(i=0; i<totseqs; i++) seq[i][l] = 0;
 fini:
 *pheader = NULL;
@@ -986,7 +1012,7 @@ return retval;
 
 int save_fasta_file(const char *fname, char **seq, char **comments,
 	char **seqname, int totseqs, int *eachlength, region *region_used,
-	int *sel_seqs, int tot_sel_seqs)
+	int *sel_seqs, int tot_sel_seqs, int spaces_in_names)
 /* sauver des sequences ou des regions au format fasta
 region_used pointe vers la liste des regions a sauver
 si region_used == NULL, toutes les sequences sont sauvees
@@ -1019,12 +1045,17 @@ for(num=0; num<totseqs; num++) {
 	all_sequence.fin = eachlength[num];
 	fputc('>', out);
 	p = seqname[num];
-	while(*p != 0) { fputc( *p == ' ' ? '_' : *p, out); p++; }
-	if(comments != NULL && comments[num] != NULL) {
-		putc(' ', out); 
-		p = comments[num] + 1;
-		while(*p != '\n' && *p != 0) putc(*(p++), out);
-		}
+	if(spaces_in_names) {
+	  fputs(p, out);
+	  }
+	else {
+	  while(*p != 0) { fputc( *p == ' ' ? '_' : *p, out); p++; }
+	  if(comments != NULL && comments[num] != NULL) {
+		  putc(' ', out); 
+		  p = comments[num] + 1;
+		  while(*p != '\n' && *p != 0) putc(*(p++), out);
+		  }
+	  }
 	putc('\n', out);
 	if(ferror(out)) goto fin;
 	do	{
@@ -1249,7 +1280,7 @@ char *save_alignment_or_region(const char *fname, char **seq, char **comments,
 	int numb_species_sets, int **list_species_sets, 
 	char **name_species_sets, int *sel_seqs, int tot_sel_seqs, int protein,
 	int tot_comment_lines, char **comment_name, char **comment_line, int phylipwidnames, 
-	int tot_trees, char **trees, const Fl_Menu_Item *items)
+	int tot_trees, char **trees, const Fl_Menu_Item *items, int spaces_in_fasta_names)
 /* sauver des sequences ou des regions au format de fichier format
 region_used pointe vers la liste des regions a sauver
 si region_used == NULL, toutes les sequences entieres sont sauvees
@@ -1289,7 +1320,7 @@ else if(format == MSF_FORMAT)
 else if(format == FASTA_FORMAT) 
 	err = save_fasta_file(fname, seq, comments,
 		seqname, totseqs, eachlength, region_used,
-		sel_seqs, tot_sel_seqs);
+		sel_seqs, tot_sel_seqs, spaces_in_fasta_names);
 if(err == 0)
 	return NULL;
 else if(err == 1) 
@@ -1513,7 +1544,7 @@ static char seaview_prog_dir[200] = "";
 void inform_prog_dir(const char *arg0)
 {
 char *p;
-if((p = strrchr(arg0, '/')) != NULL) {
+if((p = (char*)strrchr(arg0, '/')) != NULL) {
 	memcpy(seaview_prog_dir, arg0, p - arg0 + 1);
 	seaview_prog_dir[p - arg0 + 1] = 0;
 	}
@@ -1725,7 +1756,14 @@ char *p, types_list[500] = "";
 Fl_Native_File_Chooser *chooser = new Fl_Native_File_Chooser();
 chooser->type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);   
 chooser->title(message);  
-chooser->preset_file(fname);   
+chooser->preset_file(fname);  
+#ifndef MICRO
+	p = (char *)chooser->showChoice->label();
+	if(p != NULL && strcmp(p, Fl_File_Chooser::show_label) == 0) {
+	   chooser->showChoice->label("Format:");
+	   chooser->showChoice->labelsize( chooser->showChoice->labelsize() - 1);
+	   }
+#endif
 	
 p = types_list;
 for(int f = 0; f < nbr_formats; f++) {
@@ -1749,7 +1787,7 @@ const char *extract_dirname(const char *pathname)
 #if defined(WIN32)
 	p = strrchr(pathname,'\\'); 
 #else
-	p = strrchr(pathname,'/'); 
+	p = (char*)strrchr(pathname,'/'); 
 #endif
 	if(p == NULL) dirname[0] = 0;
 	else {
