@@ -2,7 +2,7 @@
 #include <FL/Fl_Hold_Browser.H>
 #include <ctype.h>
 
-char *translate_with_gaps(char *seq, int gc, int *warn);
+char *translate_with_gaps(char *seq, int gc);
 char *back_translate_with_gaps(char *prot, char *dna);
 void free_col_seqs(SEA_VIEW * view);
 int get_ncbi_gc_from_comment(char *comment);
@@ -25,7 +25,7 @@ extern void set_and_show_new_cursor_site(SEA_VIEW *view, int new_pos, int center
 
 void *set_viewasprots(SEA_VIEW * view, int onoff)
 {
-int num, oldmodif, gc, oldtotsel, *oldselseqs, oldsites, taille, oldcursorseq, oldcursorsite, warn;
+int num, oldmodif, gc, oldtotsel, *oldselseqs, oldsites, taille, oldcursorseq, oldcursorsite;
 char **protseq, **dnaseq, **olddna, **retval, *masename;
 
 oldmodif = view->modif_but_not_saved;
@@ -39,10 +39,9 @@ get_menu_taille_valeur( (Fl_Menu_ *)(view->menu_regions), &taille, &oldsites);
 if(onoff) { /* ON */
 	protseq = (char **)malloc(view->tot_seqs * sizeof(char *));
 	if(protseq == NULL) return NULL;
-	warn = TRUE;
 	for(num=0; num < view->tot_seqs; num++) {
 		gc = (view->comments != NULL ? get_ncbi_gc_from_comment(view->comments[num]) : 1);
-		protseq[num] = translate_with_gaps(view->sequence[num], get_acnuc_gc_number(gc), &warn);
+		protseq[num] = translate_with_gaps(view->sequence[num], get_acnuc_gc_number(gc));
 		if(protseq[num] == NULL) {
 			while(--num >= 0) free(protseq[num]);
 			free(protseq);
@@ -119,108 +118,62 @@ return retval;
 }
 
 
-char *translate_with_gaps(char *seq, int gc, int *warn)
+char *translate_with_gaps(char *seq, int gc)
 {
-int gaps, lp, lc, l = strlen(seq), first = TRUE;
-char *p, codon[4], aa, *oldseq;
-
-lc = 0;
-lp = 0; gaps = 0;
-p = (char *)malloc(l/3 + 5);
-if(p == NULL) return NULL;
-oldseq = seq;
---seq;
-while(*(++seq) != 0) {
-	if(*seq == '-') {
-		gaps++;
-		if(gaps == 3) { p[lp++] = '-'; gaps = 0; }
-		continue;
-		}
-	if(first) { /* if 1st codon is incomplete, put @ at missing gap positions */
-		first = FALSE;
-		while(gaps > 0) {
-			*(--seq) = '@';
-			gaps--;
+	int lp, l = strlen(seq);
+	char *p, *last = seq + l;
+	lp = 0; 
+	p = (char *)malloc(l/3 + 5);
+	if(p == NULL) return NULL;
+	while(seq + 2 < last) {
+		if(strncmp(seq, "---", 3) == 0) {
+			p[lp++] = '-';
 			}
-		}
-	else if(gaps > 0 && *warn) {
-		*warn = FALSE;
-		l = fl_choice("This alignment contains gaps of length not multiple of 3.\n"
-					  "These gaps will be altered if you display your data as protein sequences.\n"
-					  "Please choose:", "Keep DNA view", "View as proteins", NULL);
-		if(l == 0) {
-			while(*oldseq == '-') oldseq++;
-			while(*oldseq == '@') *(oldseq++) = '-'; //remove added @ if partial first codon
-			free(p); 
-			return NULL; 
+		else {
+			if(*seq == '-') *seq = '@';
+			if(*(seq + 1) == '-') *(seq + 1) = '@';
+			if(*(seq + 2) == '-') *(seq + 2) = '@';
+			p[lp++] = codaa(seq, gc);
 			}
+		seq += 3;
 		}
-	codon[lc++] = *seq;
-	if(lc >= 3) {
-		aa = codaa(codon, gc);
-		p[lp++] = aa;
-		lc = 0;
-		}
-	}
-p[lp] = 0;
-if(lc > 0) { /* add 'X' for partial final codon */
-	l = lp - 1;
-	while(l > 0 && p[l] == '-') l--;
-	memmove(p + l + 2, p + l + 1, lp - l);
-	p[l + 1] = 'X';
-	}
-else if(gaps > 0) { /* add '-' for partial remaining gaps */
-	p[lp++] = '-';
 	p[lp] = 0;
-	}
-return p;
+	return p;
 }
 
 
 char *back_translate_with_gaps(char *prot, char *dna)
 {
-int l;
-char *newdna, *pp, *pd;
-
-l = strlen(prot);
-newdna = (char *)malloc(3*l + 7);
-if(newdna == NULL) return NULL;
-pp = prot - 1; pd = newdna;
-while(*(++pp) != 0) {
-	if(*pp == '-') {
-		memcpy(pd, "---", 3);
-		pd += 3;
-		}
-	else	{
-		while(*dna == '-') dna++;
-		if(*dna == 0) {
+	int l;
+	char *newdna, *pd;
+	
+	l = strlen(prot);
+	newdna = (char *)malloc(3*l + 7);
+	if(newdna == NULL) return NULL;
+	pd = newdna;
+	prot--;
+	while(*(++prot) != 0) {
+		while(memcmp(dna, "---", 3) == 0) dna += 3;
+		if(*prot == '-') {
 			memcpy(pd, "---", 3);
 			pd += 3;
-			continue;
 			}
-		*(pd++) = *(dna++);
-		while(*dna == '-') dna++;
-		if(*dna == 0) {
-			memcpy(pd, "--", 2);
-			pd += 2;
-			continue;
+		else {
+			if(*dna == '@') *dna = '-';
+			if(*(dna + 1) == '@') *(dna + 1) = '-';
+			if(*(dna + 2) == '@') *(dna + 2) = '-';
+			memcpy(pd, dna, 3);
+			pd += 3;
+			dna += 3;
 			}
-		*(pd++) = *(dna++);
-		while(*dna == '-') dna++;
-		if(*dna == 0) {
-			*(pd++) = '-';
-			continue;
-			}
-		*(pd++) = *(dna++);
 		}
-	}
-*pd = 0;
-pd = newdna;
-/* remove 1 or 2 @ added by translate_with_gaps for partial first codon */
-while(*pd == '-') pd++;
-if(*pd == '@') *(pd++) = '-';
-if(*pd == '@') *pd = '-';	
-return newdna;
+	*pd = 0;
+	while(memcmp(dna, "---", 3) == 0) dna += 3;
+	if(*dna != 0) {//if a partial codon remains at end
+		while(*(pd-1) == '-') pd--;
+		strcpy(pd, dna);
+		}
+	return newdna;
 }
 
 
