@@ -25,12 +25,6 @@ char **getutf8args(int *pargc);
 void draw_rotated_string(const char *strDraw, int x, int y, float degrees /*clockwise*/);
 
 /* extern vars & functions */
-extern HDC fl_gc;
-#if FL_MAJOR_VERSION*100 + FL_MINOR_VERSION >= 103
-extern Fl_Font_Descriptor* fl_fontsize;
-#else
-extern Fl_FontSize* fl_fontsize;
-#endif
 extern char *wchartoutf8(const wchar_t *in);
 
 void MG_fl_draw(const char *txt, int x, int y)
